@@ -33,6 +33,12 @@ Bool_t TDirectory::fgAddDirectory = kTRUE;
 
 const Int_t  kMaxLen = 2048;
 
+static std::atomic_flag *GetCurrentDirectoryLock()
+{
+   thread_local std::atomic_flag gDirectory_lock = ATOMIC_FLAG_INIT;
+   return &gDirectory_lock;
+}
+
 /** \class TDirectory
 \ingroup Base
 
@@ -90,7 +96,9 @@ TDirectory::TDirectory(const char *name, const char *title, Option_t * /*classna
 
 TDirectory::~TDirectory()
 {
-   if (!gROOT) {
+   // Use gROOTLocal to avoid triggering undesired initialization of gROOT.
+   // For example in compiled C++ programs that don't use it directly.
+   if (!ROOT::Internal::gROOTLocal) {
       delete fList;
       return; //when called by TROOT destructor
    }
@@ -207,7 +215,10 @@ void TDirectory::Append(TObject *obj, Bool_t replace /* = kFALSE */)
    }
 
    fList->Add(obj);
-   obj->SetBit(kMustCleanup);
+   // A priori, a `TDirectory` object is assumed to not have shared ownership.
+   // If it is, let's rely on the user to update the bit.
+   if (!dynamic_cast<TDirectory*>(obj))
+      obj->SetBit(kMustCleanup);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -262,7 +273,12 @@ void TDirectory::CleanTargets()
          const auto ctxt = fContext;
          ctxt->fDirectoryWait = true;
 
-         ctxt->fDirectory = nullptr;
+         // If fDirectory is assigned to gROOT but we do not unregister ctxt
+         // (and/or stop unregister for gROOT) then ~TContext will call Unregister on gROOT.
+         // Then unregister of this ctxt and its Previous context can actually be run
+         // in parallel (this takes the gROOT lock, Previous takes the lock of fDirectory)
+         // and thus step on each other.
+         ctxt->fDirectory = nullptr; // Can not be gROOT
 
          if (ctxt->fActiveDestructor) {
             extraWait.push_back(fContext);
@@ -272,9 +288,30 @@ void TDirectory::CleanTargets()
          fContext = next;
       }
 
-      for(auto ptr : fGDirectories) {
+      // Now loop through the set of thread local 'gDirectory' that
+      // have a one point or another pointed to this directory.
+      for (auto &ptr : fGDirectories) {
+         // If the thread local gDirectory still point to this directory
+         // we need to reset it using the following sematic:
+         // we fall back to the mother/owner of this directory or gROOTLocal
+         // if there is no parent or nullptr if the current object is gROOTLocal.
          if (ptr->load() == this) {
-            (*ptr) = nullptr;
+            TDirectory *next = GetMotherDir();
+            if (!next || next == this) {
+               if (this == ROOT::Internal::gROOTLocal) { /// in that case next == this.
+                  next = nullptr;
+               } else {
+                  next = ROOT::Internal::gROOTLocal;
+               }
+            } else {
+               // We can not use 'cd' as this would access the current thread
+               // rather than the thread corresponding to that gDirectory.
+               next->RegisterGDirectory(ptr);
+            }
+            // Actually do the update of the thread local gDirectory
+            // using its object specific lock.
+            auto This = this;
+            ptr->compare_exchange_strong(This, next);
          }
       }
    }
@@ -282,21 +319,13 @@ void TDirectory::CleanTargets()
       // Wait until the TContext is done spinning
       // over the lock.
       while(context->fActiveDestructor);
+      // And now let the TContext destructor finish.
       context->fDirectoryWait = false;
    }
 
-   if (gDirectory == this) {
-      TDirectory *cursav = GetMotherDir();
-      if (cursav && cursav != this) {
-         cursav->cd();
-      } else {
-         if (this == gROOT) {
-            gDirectory = nullptr;
-         } else {
-            gROOT->cd();
-         }
-      }
-   }
+   // Wait until all register attempts are done.
+   while(fContextPeg) {}
+
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -318,7 +347,7 @@ static TBuffer* R__CreateBuffer()
    Int_t size = 10000;
    void *args[] = { &mode, &size };
    TBuffer *result;
-   creator(0,2,args,&result);
+   creator(nullptr,2,args,&result);
    return result;
 }
 
@@ -381,16 +410,28 @@ TObject *TDirectory::CloneObject(const TObject *obj, Bool_t autoadd /* = kTRUE *
    return newobj;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// Return the (address of) a shared pointer to the struct holding the
+/// actual thread local gDirectory pointer and the atomic_flag for its lock.
+TDirectory::SharedGDirectory_t &TDirectory::GetSharedLocalCurrentDirectory()
+{
+   using shared_ptr_type = TDirectory::SharedGDirectory_t;
+
+   // Note in previous implementation every time gDirectory was lookup in
+   // a thread, if it was set to nullptr it would be reset to gROOT.  This
+   // was unexpected and this routine is not re-introducing this issue.
+   thread_local shared_ptr_type currentDirectory =
+      std::make_shared<shared_ptr_type::element_type>(ROOT::Internal::gROOTLocal);
+
+   return currentDirectory;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Return the current directory for the current thread.
 
 std::atomic<TDirectory*> &TDirectory::CurrentDirectory()
 {
-   static std::atomic<TDirectory*> currentDirectory{nullptr};
-   if (!gThreadTsd)
-      return currentDirectory;
-   else
-      return *(std::atomic<TDirectory*>*)(*gThreadTsd)(&currentDirectory,ROOT::kDirectoryThreadSlot);
+   return *GetSharedLocalCurrentDirectory().get();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -413,7 +454,7 @@ TDirectory *TDirectory::GetDirectory(const char *apath,
       return this;
    }
 
-   if (funcname==0 || strlen(funcname)==0) funcname = "GetDirectory";
+   if (funcname==nullptr || strlen(funcname)==0) funcname = "GetDirectory";
 
    TDirectory *result = this;
 
@@ -492,6 +533,23 @@ TDirectory *TDirectory::GetDirectory(const char *apath,
 ////////////////////////////////////////////////////////////////////////////////
 /// Change current directory to "this" directory.
 ///
+/// Returns kTRUE (it's guaranteed to succeed).
+
+Bool_t TDirectory::cd()
+{
+   auto &thread_local_gdirectory = GetSharedLocalCurrentDirectory();
+
+   RegisterGDirectory(thread_local_gdirectory);
+
+   thread_local_gdirectory->exchange(this);
+
+   return kTRUE;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+/// Change current directory to "this" directory or to the directory described
+/// by the path if given one.
+///
 /// Using path one can change the current directory to "path". The absolute path
 /// syntax is: `file.root:/dir1/dir2`
 /// where `file.root` is the file and `/dir1/dir2` the desired subdirectory
@@ -507,7 +565,8 @@ Bool_t TDirectory::cd(const char *path)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Change current directory to "this" directory.
+/// Change current directory to "this" directory or to the directory described
+/// by the path if given one.
 ///
 /// Using path one can
 /// change the current directory to "path". The absolute path syntax is:
@@ -521,16 +580,10 @@ Bool_t TDirectory::cd(const char *path)
 
 Bool_t TDirectory::cd1(const char *apath)
 {
-   Int_t nch = 0;
-   if (apath) nch = strlen(apath);
-   if (!nch) {
-      auto &global = CurrentDirectory();
-      RegisterGDirectory(&global);
-      global = this;
-      return kTRUE;
-   }
+   if (!apath || !apath[0])
+      return this->cd();
 
-   TDirectory *where = GetDirectory(apath,kTRUE,"cd");
+   TDirectory *where = GetDirectory(apath, kTRUE, "cd");
    if (where) {
       where->cd();
       return kTRUE;
@@ -557,17 +610,17 @@ Bool_t TDirectory::Cd(const char *path)
 /// `file.root:/dir1/dir2`
 /// where file.root is the file and `/dir1/dir2` the desired subdirectory
 /// in the file.
+/// Relative syntax is relative to the current directory `gDirectory`, e.g.: `../aa`.
 ///
 /// Returns kFALSE in case path does not exist.
 
 Bool_t TDirectory::Cd1(const char *apath)
 {
    // null path is always true (i.e. stay in the current directory)
-   Int_t nch = 0;
-   if (apath) nch = strlen(apath);
-   if (!nch) return kTRUE;
+   if (!apath || !apath[0])
+      return kTRUE;
 
-   TDirectory *where = gDirectory->GetDirectory(apath,kTRUE,"Cd");
+   TDirectory *where = gDirectory->GetDirectory(apath, kTRUE, "Cd");
    if (where) {
       where->cd();
       return kTRUE;
@@ -581,7 +634,6 @@ Bool_t TDirectory::Cd1(const char *apath)
 void TDirectory::Clear(Option_t *)
 {
    if (fList) fList->Clear();
-
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -673,7 +725,7 @@ void TDirectory::Delete(const char *namecycle)
    if(strcmp(name,"*") == 0)   deleteall = 1;
    if(strcmp(name,"*T") == 0){ deleteall = 1; deletetree = 1;}
    if(strcmp(name,"T*") == 0){ deleteall = 1; deletetree = 1;}
-   if(namecycle==0 || !namecycle[0]){ deleteall = 1; deletetree = 1;}
+   if(namecycle==nullptr || !namecycle[0]){ deleteall = 1; deletetree = 1;}
    TRegexp re(name,kTRUE);
    TString s;
    Int_t deleteOK = 0;
@@ -1056,7 +1108,6 @@ TDirectory *TDirectory::mkdir(const char *name, const char *title, Bool_t return
    }
    if (!name || !title || !name[0]) return nullptr;
    if (!title[0]) title = name;
-   TDirectory *newdir = nullptr;
    if (const char *slash = strchr(name,'/')) {
       Long_t size = Long_t(slash-name);
       char *workname = new char[size+1];
@@ -1068,15 +1119,12 @@ TDirectory *TDirectory::mkdir(const char *name, const char *title, Bool_t return
          tmpdir = mkdir(workname,title);
       delete[] workname;
       if (!tmpdir) return nullptr;
-      if (!newdir) newdir = tmpdir;
       return tmpdir->mkdir(slash+1);
    }
 
    TDirectory::TContext ctxt(this);
 
-   newdir = new TDirectory(name, title, "", this);
-
-   return newdir;
+   return new TDirectory(name, title, "", this);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1154,7 +1202,8 @@ void TDirectory::pwd() const
 
 void TDirectory::RecursiveRemove(TObject *obj)
 {
-   fList->RecursiveRemove(obj);
+   if (fList)
+      fList->RecursiveRemove(obj);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1162,7 +1211,7 @@ void TDirectory::RecursiveRemove(TObject *obj)
 
 TObject *TDirectory::Remove(TObject* obj)
 {
-   TObject *p = 0;
+   TObject *p = nullptr;
    if (fList) {
       p = fList->Remove(obj);
    }
@@ -1177,7 +1226,7 @@ TObject *TDirectory::Remove(TObject* obj)
 
 void TDirectory::rmdir(const char *name)
 {
-   if ((name==0) || (*name==0)) return;
+   if ((name==nullptr) || (*name==0)) return;
 
    TString mask(name);
    mask+=";*";
@@ -1206,7 +1255,7 @@ Int_t TDirectory::SaveObjectAs(const TObject *obj, const char *filename, Option_
    }
    TString cmd;
    if (fname.Index(".json") > 0) {
-      cmd.Form("TBufferJSON::ExportToFile(\"%s\",(TObject*) %s, \"%s\");", fname.Data(), TString::LLtoa((Long_t)obj, 10).Data(), (option ? option : ""));
+      cmd.Form("TBufferJSON::ExportToFile(\"%s\",(TObject*) %s, \"%s\");", fname.Data(), TString::LLtoa((Longptr_t)obj, 10).Data(), (option ? option : ""));
       nbytes = gROOT->ProcessLine(cmd);
    } else {
       cmd.Form("TFile::Open(\"%s\",\"recreate\");",fname.Data());
@@ -1239,26 +1288,44 @@ void TDirectory::SetName(const char* newname)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Encode the name and cycle into buffer like: "aap;2".
+/// @note if cycle is `9999`, the name is copied to the buffer without any `;`
 
 void TDirectory::EncodeNameCycle(char *buffer, const char *name, Short_t cycle)
 {
    if (cycle == 9999)
       strcpy(buffer, name);
-   else
-      sprintf(buffer, "%s;%d", name, cycle);
+   else {
+      // sizeof(buffer) is unknown, this interface is broken.
+      // It's now also deprecated. Until it's removed, silence sprintf warning
+      // on macOS / Xcode / clang by using an equivalently bad snprintf without
+      // knowledge of sizeof(buffer):
+      size_t unsafeSize = strlen(name) + std::to_string(cycle).length() + 1;
+      snprintf(buffer, unsafeSize, "%s;%d", name, cycle);
+   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Decode a namecycle "aap;2" into name "aap" and cycle "2". Destination
 /// buffer size for name (including string terminator) should be specified in
 /// namesize.
+/// @note Edge cases:
+///   - If the number after the `;` is larger than `SHORT_MAX`, cycle is set to `0`.
+///   - If name ends with `;*`, cycle is set to 10000`.
+///   - In all other cases, i.e. when number is not a digit or buffer is a nullptr, cycle is set to `9999`.
 
 void TDirectory::DecodeNameCycle(const char *buffer, char *name, Short_t &cycle,
                                  const size_t namesize)
 {
+   if (!buffer) {
+      ::Warning("TDirectory::DecodeNameCycle",
+         "An empty buffer was passed, cannot decode.");
+      cycle = 9999;
+      return;
+   }
+
    size_t len = 0;
    const char *ni = strchr(buffer, ';');
-
+   
    if (ni) {
       // Found ';'
       len = ni - buffer;
@@ -1273,7 +1340,7 @@ void TDirectory::DecodeNameCycle(const char *buffer, char *name, Short_t &cycle,
       if (len > namesize-1ul) len = namesize-1;  // accommodate string terminator
    } else {
       ::Warning("TDirectory::DecodeNameCycle",
-         "Using unsafe version: invoke this metod by specifying the buffer size");
+         "Using unsafe version: invoke this method by specifying the buffer size");
    }
 
    strncpy(name, buffer, len);
@@ -1291,12 +1358,32 @@ void TDirectory::DecodeNameCycle(const char *buffer, char *name, Short_t &cycle,
       cycle = 9999;
 }
 
+void TDirectory::TContext::RegisterCurrentDirectory()
+{
+   // peg the current directory
+   TDirectory *current;
+   {
+      ROOT::Internal::TSpinLockGuard slg(*GetCurrentDirectoryLock());
+      current = TDirectory::CurrentDirectory().load();
+      // Don't peg if there is no current directory or if the current
+      // directory's destruction has already started (in another thread)
+      // and is waiting for this thread to leave the critical section.
+      if (!current || !current->IsBuilt())
+         return;
+      ++(current->fContextPeg);
+   }
+   current->RegisterContext(this);
+   --(current->fContextPeg);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /// Register a TContext pointing to this TDirectory object
 
 void TDirectory::RegisterContext(TContext *ctxt) {
    ROOT::Internal::TSpinLockGuard slg(fSpinLock);
 
+   if (!IsBuilt() || this == ROOT::Internal::gROOTLocal)
+      return;
    if (fContext) {
       TContext *current = fContext;
       while(current->fNext) {
@@ -1312,12 +1399,13 @@ void TDirectory::RegisterContext(TContext *ctxt) {
 ////////////////////////////////////////////////////////////////////////////////
 /// Register a std::atomic<TDirectory*> that will soon be pointing to this TDirectory object
 
-void TDirectory::RegisterGDirectory(std::atomic<TDirectory*> *globalptr)
+void TDirectory::RegisterGDirectory(TDirectory::SharedGDirectory_t &gdirectory_ptr)
 {
    ROOT::Internal::TSpinLockGuard slg(fSpinLock);
-
-   if (std::find(fGDirectories.begin(), fGDirectories.end(), globalptr) != fGDirectories.end())
-      fGDirectories.push_back(globalptr);
+   if (std::find(fGDirectories.begin(), fGDirectories.end(), gdirectory_ptr) == fGDirectories.end()) {
+      fGDirectories.emplace_back(gdirectory_ptr);
+   }
+   // FIXME:
    // globalptr->load()->fGDirectories will still contain globalptr, but we cannot
    // know whether globalptr->load() has been deleted by another thread in the meantime.
 }
@@ -1343,7 +1431,7 @@ void TDirectory::UnregisterContext(TContext *ctxt) {
    ROOT::Internal::TSpinLockGuard slg(fSpinLock);
 
    // Another thread already unregistered the TContext.
-   if (ctxt->fDirectory == nullptr)
+   if (ctxt->fDirectory == nullptr || ctxt->fDirectory == ROOT::Internal::gROOTLocal)
       return;
 
    if (ctxt==fContext) {
