@@ -19,7 +19,11 @@
 #include "TGeoTrack.h"
 
 /** \class TGeoTrack
-\ingroup Geometry_classes
+\ingroup Geometry_painter
+
+\deprecated
+Use of TGeoTrack is deprecated. For the event display please switch to TEve (using TEveTracks
+or TEvePointSet to display tracks specifically) or to [REve](https://root.cern/doc/master/tracks_8C_source.html).
 
 Class for user-defined tracks attached to a geometry.
 Tracks are 3D objects made of points and they store a
@@ -131,7 +135,8 @@ void TGeoTrack::AnimateTrack(Double_t tmin, Double_t tmax, Double_t nframes, Opt
    box[3] = box[4] = box[5] = 100;
    gGeoManager->SetTminTmax(0,0);
    Draw(opt.Data());
-   Double_t start[6], end[6];
+   Double_t start[6] = {0,0,0,0,0,0};
+   Double_t end[6] = {0,0,0,0,0,0};
    Int_t i, j;
    Double_t dlat=0, dlong=0, dpsi=0;
    Double_t dd[6] = {0,0,0,0,0,0};
