@@ -14,7 +14,6 @@
 #include "ROOT/REveProjections.hxx"
 #include "ROOT/REveProjectionManager.hxx"
 #include "ROOT/REveRGBAPalette.hxx"
-//#include "ROOT/REveText.hxx"
 #include "ROOT/REveRenderData.hxx"
 #include "ROOT/REveTrans.hxx"
 
@@ -23,11 +22,11 @@
 #include "TMath.h"
 #include "TAxis.h"
 
-// #include "TGLUtil.h"
-
 #include <cassert>
 #include <iostream>
 
+#include <nlohmann/json.hpp>
+
 using namespace ROOT::Experimental;
 
 /** \class REveCaloViz
@@ -603,6 +602,8 @@ void REveCalo3D::MakeBarrelCell(const REveCaloData::CellGeom_t &cellData, float
 void REveCalo3D::BuildRenderData()
 {
    AssertCellIdCache();
+   fRenderData = std::make_unique<REveRenderData>("makeCalo3D");
+   
    if (fCellList.empty())
    return;
 
@@ -612,7 +613,6 @@ void REveCalo3D::BuildRenderData()
    Int_t   prevTower = -1;
    Float_t offset = 0;
 
-   fRenderData = std::make_unique<REveRenderData>("makeCalo3D");
    float pnts[24];
    for (REveCaloData::vCellId_i i = fCellList.begin(); i != fCellList.end(); ++i)
    {
@@ -1173,6 +1173,8 @@ void REveCalo2D::WriteCoreJsonSelection(nlohmann::json &j, REveCaloData::vCellId
 void REveCalo2D::BuildRenderData()
 {
    AssertCellIdCache();
+   fRenderData = std::make_unique<REveRenderData>("makeCalo2D");
+   
    bool isEmpty = fData->Empty();
 
    for (vBinCells_i it = fCellLists.begin(); it != fCellLists.end(); ++it)
@@ -1185,8 +1187,6 @@ void REveCalo2D::BuildRenderData()
    }
    if (isEmpty) return;
 
-   fRenderData = std::make_unique<REveRenderData>("makeCalo2D");
-
    if (IsRPhi())
       BuildRenderDataRPhi();
    else
