@@ -34,7 +34,7 @@
 ClassImp(TGraphAsymmErrors);
 
 /** \class TGraphAsymmErrors
-    \ingroup Hist
+    \ingroup Graphs
 TGraph with asymmetric error bars.
 
 The TGraphAsymmErrors painting is performed thanks to the TGraphPainter
@@ -94,7 +94,7 @@ TGraphAsymmErrors::TGraphAsymmErrors(const TGraphAsymmErrors &gr)
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// TGraphAsymmErrors assignment operator
+/// TGraphAsymmErrors assignment operator.
 
 TGraphAsymmErrors& TGraphAsymmErrors::operator=(const TGraphAsymmErrors &gr)
 {
@@ -448,14 +448,15 @@ Double_t** TGraphAsymmErrors::Allocate(Int_t size) {
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Apply a function to all data points `y = f(x,y)`
+/// Apply a function to all data points \f$ y = f(x,y) \f$
 ///
-/// Errors are calculated as `eyh = f(x,y+eyh)-f(x,y)` and
-/// `eyl = f(x,y)-f(x,y-eyl)`
+/// Errors are calculated as \f$ eyh = f(x,y+eyh)-f(x,y) \f$ and
+/// \f$ eyl = f(x,y)-f(x,y-eyl) \f$
 ///
 /// Special treatment has to be applied for the functions where the
 /// role of "up" and "down" is reversed.
-/// function suggested/implemented by Miroslav Helbich <helbich@mail.desy.de>
+///
+/// Function suggested/implemented by Miroslav Helbich <helbich@mail.desy.de>
 
 void TGraphAsymmErrors::Apply(TF1 *f)
 {
@@ -965,7 +966,7 @@ void TGraphAsymmErrors::Divide(const TH1* pass, const TH1* total, Option_t *opt)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Compute Range
+/// Compute Range.
 
 void TGraphAsymmErrors::ComputeRange(Double_t &xmin, Double_t &ymin, Double_t &xmax, Double_t &ymax) const
 {
@@ -1020,8 +1021,8 @@ void TGraphAsymmErrors::CopyAndRelease(Double_t **newarrays,
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Copy errors from fE*** to arrays[***]
-/// or to f*** Copy points.
+/// Copy errors from `fE***` to `arrays[***]`
+/// or to `f***` Copy points.
 
 Bool_t TGraphAsymmErrors::CopyPoints(Double_t **arrays,
                                      Int_t ibegin, Int_t iend, Int_t obegin)
@@ -1047,7 +1048,7 @@ Bool_t TGraphAsymmErrors::CopyPoints(Double_t **arrays,
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Should be called from ctors after fNpoints has been set
+/// Should be called from ctors after `fNpoints` has been set.
 /// Note: This function should be called only from the constructor
 /// since it does not delete previously existing arrays
 
@@ -1065,7 +1066,7 @@ Bool_t TGraphAsymmErrors::CtorAllocate(void)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-///  protected function to perform the merge operation of a graph with asymmetric errors
+/// Protected function to perform the merge operation of a graph with asymmetric errors.
 
 Bool_t TGraphAsymmErrors::DoMerge(const TGraph *g)
 {
@@ -1092,7 +1093,7 @@ Bool_t TGraphAsymmErrors::DoMerge(const TGraph *g)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Set zero values for point arrays in the range [begin, end)
+/// Set zero values for point arrays in the range `[begin, end]`
 
 void TGraphAsymmErrors::FillZero(Int_t begin, Int_t end,
                                  Bool_t from_ctor)
@@ -1109,7 +1110,6 @@ void TGraphAsymmErrors::FillZero(Int_t begin, Int_t end,
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// This function is called by GraphFitChisquare.
 /// It returns the error along X at point i.
 
 Double_t TGraphAsymmErrors::GetErrorX(Int_t i) const
@@ -1124,7 +1124,6 @@ Double_t TGraphAsymmErrors::GetErrorX(Int_t i) const
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// This function is called by GraphFitChisquare.
 /// It returns the error along Y at point i.
 
 Double_t TGraphAsymmErrors::GetErrorY(Int_t i) const
@@ -1184,7 +1183,7 @@ Double_t TGraphAsymmErrors::GetErrorYlow(Int_t i) const
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Adds all graphs with asymmetric errors from the collection to this graph.
-/// Returns the total number of poins in the result or -1 in case of an error.
+/// Returns the total number of points in the result or -1 in case of an error.
 
 Int_t TGraphAsymmErrors::Merge(TCollection* li)
 {
@@ -1229,7 +1228,7 @@ void TGraphAsymmErrors::Print(Option_t *) const
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Save primitive as a C++ statement(s) on output stream out
+/// Save primitive as a C++ statement(s) on output stream out.
 
 void TGraphAsymmErrors::SavePrimitive(std::ostream &out, Option_t *option /*= ""*/)
 {
@@ -1315,6 +1314,35 @@ void TGraphAsymmErrors::SavePrimitive(std::ostream &out, Option_t *option /*= ""
    }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// Multiply the values and errors of a TGraphAsymmErrors by a constant c1.
+///
+/// If option contains "x" the x values and errors are scaled
+/// If option contains "y" the y values and errors are scaled
+/// If option contains "xy" both x and y values and errors are scaled
+
+void TGraphAsymmErrors::Scale(Double_t c1, Option_t *option)
+{
+   TGraph::Scale(c1, option);
+   TString opt = option; opt.ToLower();
+   if (opt.Contains("x") && GetEXlow()) {
+      for (Int_t i=0; i<GetN(); i++)
+         GetEXlow()[i] *= c1;
+   }
+   if (opt.Contains("x") && GetEXhigh()) {
+      for (Int_t i=0; i<GetN(); i++)
+         GetEXhigh()[i] *= c1;
+   }
+   if (opt.Contains("y") && GetEYlow()) {
+      for (Int_t i=0; i<GetN(); i++)
+         GetEYlow()[i] *= c1;
+   }
+   if (opt.Contains("y") && GetEYhigh()) {
+      for (Int_t i=0; i<GetN(); i++)
+         GetEYhigh()[i] *= c1;
+   }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Set ex and ey values for point pointed by the mouse.
 
@@ -1360,7 +1388,7 @@ void TGraphAsymmErrors::SetPointError(Int_t i, Double_t exl, Double_t exh, Doubl
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Set EXlow for point i
+/// Set EXlow for point `i`.
 
 void TGraphAsymmErrors::SetPointEXlow(Int_t i, Double_t exl)
 {
@@ -1374,7 +1402,7 @@ void TGraphAsymmErrors::SetPointEXlow(Int_t i, Double_t exl)
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Set EXhigh for point i
+/// Set EXhigh for point `i`.
 
 void TGraphAsymmErrors::SetPointEXhigh(Int_t i, Double_t exh)
 {
@@ -1388,7 +1416,7 @@ void TGraphAsymmErrors::SetPointEXhigh(Int_t i, Double_t exh)
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Set EYlow for point i
+/// Set EYlow for point `i`.
 
 void TGraphAsymmErrors::SetPointEYlow(Int_t i, Double_t eyl)
 {
@@ -1402,7 +1430,7 @@ void TGraphAsymmErrors::SetPointEYlow(Int_t i, Double_t eyl)
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Set EYhigh for point i
+/// Set EYhigh for point `i`.
 
 void TGraphAsymmErrors::SetPointEYhigh(Int_t i, Double_t eyh)
 {
