@@ -330,9 +330,9 @@ void RooPlot::initialize()
 TString RooPlot::histName() const
 {
   if (_plotVarClone) {
-    return TString(Form("frame_%s_%lx",_plotVarClone->GetName(),(ULong_t)this)) ;
+    return TString(Form("frame_%s_%zx",_plotVarClone->GetName(),(size_t)this)) ;
   } else {
-    return TString(Form("frame_%lx",(ULong_t)this)) ;
+    return TString(Form("frame_%zx",(size_t)this)) ;
   }
 }
 
@@ -383,36 +383,6 @@ void RooPlot::updateNormVars(const RooArgSet &vars)
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-/// A plot object is a frame without any bin contents of its own so this
-/// method always returns zero.
-
-Stat_t RooPlot::GetBinContent(Int_t /*i*/) const {
-  return 0;
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// A plot object is a frame without any bin contents of its own so this
-/// method always returns zero.
-
-Stat_t RooPlot::GetBinContent(Int_t, Int_t) const
-{
-  return 0;
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// A plot object is a frame without any bin contents of its own so this
-/// method always returns zero.
-
-Stat_t RooPlot::GetBinContent(Int_t, Int_t, Int_t) const
-{
-  return 0;
-}
-
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Add a generic object to this plot. The specified options will be
 /// used to Draw() this object later. The caller transfers ownership
@@ -986,8 +956,8 @@ TObject *RooPlot::findObject(const char *name, const TClass* clas) const
 
   TIterator* iter = _items.MakeIterator() ;
   while((obj=iter->Next())) {
-    if ((!name || !TString(name).CompareTo(obj->GetName())) &&
-	(!clas || (obj->IsA()==clas))) {
+    if ((!name || name[0] == '\0' || !TString(name).CompareTo(obj->GetName()))
+        && (!clas || (obj->IsA()==clas))) {
       ret = obj ;
     }
   }
@@ -1096,8 +1066,7 @@ void RooPlot::SetMinimum(Double_t minimum)
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Calculate and return reduced chi-squared of curve with given name with respect
-/// to histogram with given name.
+/// Calculate and return reduced chi-squared between a curve and a histogram.
 ///
 /// \param[in] curvename  Name of the curve or nullptr for last curve
 /// \param[in] histname   Name of the histogram to compare to or nullptr for last added histogram
@@ -1105,8 +1074,11 @@ void RooPlot::SetMinimum(Double_t minimum)
 /// number. This means that the curve was fitted to the data with nFitParam floating
 /// parameters, which needs to be reflected in the calculation of \f$\chi^2 / \mathrm{ndf}\f$.
 /// 
-/// \return \f$ \chi^2 / \mathrm{ndf} \f$
-
+/// \return \f$ \chi^2 / \mathrm{ndf} \f$ between the plotted curve and the data.
+///
+/// \note The \f$ \chi^2 \f$ is calculated between a *plot of the original distribution* and the data.
+/// It therefore has more rounding errors than directly calculating the \f$ \chi^2 \f$ from a PDF or
+/// function. To do this, use RooChi2Var.
 Double_t RooPlot::chiSquare(const char* curvename, const char* histname, int nFitParam) const
 {
 
@@ -1129,32 +1101,75 @@ Double_t RooPlot::chiSquare(const char* curvename, const char* histname, int nFi
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Return a RooHist (derives from TGraphAsymErrors) containing the residuals of histogram 'histname' with respect
-/// to curve 'curvename'. If normalize is true, the residuals are divided by the error
-/// of the histogram, effectively returning a pull histogram.
+/// Return a RooHist (derives from TGraphAsymErrors) containing the residuals of a histogram.
 /// The plotting range of the graph is adapted to the plotting range of the current plot.
-/// If `useAverage` is true, the histogram is compared with the curve's average values within a given bin.
-/// Otherwise, the curve is interpolated at the bin centres, which is not accurate for curved distributions.
+///
+/// \param histname Name of the data histogram.
+/// Passing an empty string or `nullptr` will create residuals of the last-plotted histogram.
+/// \param curvename Name of the curve to compare to data.
+/// Passing an empty string or `nullptr` will create residuals of the last-plotted curve.
+/// \param normalize If true, the residuals are divided by the error
+/// of the histogram, effectively returning a pull histogram.
+/// \param useAverage If true, the histogram is compared with the curve averaged in each bin.
+/// Otherwise, the curve is evaluated at the bin centres, which is not accurate for strongly curved distributions.
 RooHist* RooPlot::residHist(const char* histname, const char* curvename, bool normalize, bool useAverage) const
 {
-  // Find curve object
-  RooCurve* curve = (RooCurve*) findObject(curvename,RooCurve::Class()) ;
-  if (!curve) {
-    coutE(InputArguments) << "RooPlot::residHist(" << GetName() << ") cannot find curve" << endl ;
-    return 0 ;
+  // Find all curve objects with the name "curvename" or the name of the last
+  // plotted curve (there might be multiple in the case of multi-range fits).
+  std::vector<RooCurve *> curves;
+
+  TIter next{&_items, kIterBackward};
+  while(TObject * obj = next()) {
+    if(obj->IsA() == RooCurve::Class()) {
+      // If no curvename was passed, we take by default the last curve and all
+      // other curves that have the same name
+      if((!curvename || curvename[0] == '\0') || std::string(curvename) == obj->GetName()) {
+        curvename = obj->GetName();
+        curves.push_back(static_cast<RooCurve*>(obj));
+      }
+    }
+  }
+
+  if (curves.empty()) {
+    coutE(InputArguments) << "RooPlot::residHist(" << GetName() << ") cannot find curve" << std::endl;
+    return nullptr;
   }
 
   // Find histogram object
-  RooHist* hist = (RooHist*) findObject(histname,RooHist::Class()) ;
+  auto hist = static_cast<RooHist*>(findObject(histname,RooHist::Class()));
   if (!hist) {
-    coutE(InputArguments) << "RooPlot::residHist(" << GetName() << ") cannot find histogram" << endl ;
-    return 0 ;
+    coutE(InputArguments) << "RooPlot::residHist(" << GetName() << ") cannot find histogram" << std::endl;
+    return nullptr;
   }
 
-  auto residhist = hist->makeResidHist(*curve,normalize,useAverage);
-  residhist->GetHistogram()->GetXaxis()->SetRangeUser(_hist->GetXaxis()->GetXmin(), _hist->GetXaxis()->GetXmax());
+  auto residHist = hist->createEmptyResidHist(*curves.front(), normalize);
 
-  return residhist;
+  // We consider all curves with the same name as long as the ranges don't
+  // overlap, to create also the full residual plot in multi-range fits.
+  std::vector<std::pair<double, double>> coveredRanges;
+  for(RooCurve * curve : curves) {
+    const double xmin = curve->GetPointX(0);
+    const double xmax = curve->GetPointX(curve->GetN() - 1);
+
+    for(auto const& prevRange : coveredRanges) {
+      const double pxmin = prevRange.first;
+      const double pxmax = prevRange.second;
+      // If either xmin or xmax is within any previous range, the ranges
+      // overloap and it makes to sense to also consider this curve for the
+      // residuals (i.e., they can't come from a multi-range fit).
+      if((pxmax > xmin && pxmin <= xmin) || (pxmax > xmax && pxmin <= xmax)) {
+        continue;
+      }
+    }
+
+    coveredRanges.emplace_back(xmin, xmax);
+
+    hist->fillResidHist(*residHist, *curve, normalize, useAverage);
+  }
+  residHist->GetHistogram()->GetXaxis()->SetRangeUser(_hist->GetXaxis()->GetXmin(), _hist->GetXaxis()->GetXmax());
+  residHist->GetHistogram()->GetXaxis()->SetTitle(_hist->GetXaxis()->GetTitle());
+  residHist->GetHistogram()->GetYaxis()->SetTitle(normalize ? "(Data - curve) / #sigma_{data}" : "Data - curve");
+  return residHist.release();
 }
 
 
