@@ -369,7 +369,7 @@ void RooProdGenContext::generateEvent(RooArgSet &theEvent, Int_t remaining)
 	arglv->randomize() ;
       }
     }
-    theEvent = _uniObs ;
+    theEvent.assign(_uniObs) ;
   }  
 
 }
