@@ -132,7 +132,7 @@ SPlot::SPlot():
 {
   RooArgList Args;
 
-  fSWeightVars = Args;
+  fSWeightVars.assign(Args);
 
   fSData = NULL;
 
@@ -145,7 +145,7 @@ SPlot::SPlot(const char* name, const char* title):
 {
   RooArgList Args;
 
-  fSWeightVars = Args;
+  fSWeightVars.assign(Args);
 
   fSData = NULL;
 
@@ -160,7 +160,7 @@ SPlot::SPlot(const char* name, const char* title, const RooDataSet &data):
 {
   RooArgList Args;
 
-  fSWeightVars = Args;
+  fSWeightVars.assign(Args);
 
   fSData = (RooDataSet*) &data;
 }
@@ -201,7 +201,7 @@ SPlot::SPlot(const char* name, const char* title, RooDataSet& data, RooAbsPdf* p
         const RooCmdArg& arg5, const RooCmdArg& arg6, const RooCmdArg& arg7, const RooCmdArg& arg8):
   TNamed(name, title)
 {
-  if(cloneData == 1) {
+  if(cloneData) {
     fSData = (RooDataSet*) data.Clone(newName);
     SetBit(kOwnData);
   }
@@ -471,11 +471,6 @@ void SPlot::AddSWeight( RooAbsPdf* pdf, const RooArgList &yieldsTmp,
   RooArgSet vars(*fSData->get() );
   vars.remove(projDeps, kTRUE, kTRUE);
 
-  // Attach data set
-
-  // const_cast<RooAbsPdf*>(pdf)->attachDataSet(*fSData);
-
-  pdf->attachDataSet(*fSData);
 
   // first calculate the pdf values for all species and all events
   std::vector<RooAbsRealLValue*> yieldvars ;
@@ -544,7 +539,7 @@ void SPlot::AddSWeight( RooAbsPdf* pdf, const RooArgList &yieldsTmp,
   {
     //WVE: FIX THIS PART, EVALUATION PROGRESS!!
 
-    RooStats::SetParameters(fSData->get(ievt), pdfvars);
+    pdfvars->assign(*fSData->get(ievt));
 
     for(Int_t k = 0; k < nspec; ++k) {
       auto theVar = static_cast<RooAbsRealLValue*>(yieldvars[k]);
