@@ -31,13 +31,13 @@
  Example code can be found in
   - tutorials/tree/hsimpleReader.C
   - tutorials/tree/h1analysisTreeReader.C
-  - <a href="http://root.cern.ch/gitweb?p=roottest.git;a=tree;f=root/tree/reader;hb=HEAD">This example</a>
+  - <a href="https://github.com/root-project/roottest/tree/master/root/tree/reader">This example</a>
 
  You can generate a skeleton of `TTreeReaderValue<T>` and `TTreeReaderArray<T>` declarations
  for all of a tree's branches using `TTree::MakeSelector()`.
 
  Roottest contains an
- <a href="http://root.cern.ch/gitweb?p=roottest.git;a=tree;f=root/tree/reader;hb=HEAD">example</a>
+ <a href="https://github.com/root-project/roottest/tree/master/root/tree/reader">example</a>
  showing the full power.
 
 A simpler analysis example can be found below: it histograms a function of the px and py branches.
@@ -346,7 +346,7 @@ Bool_t TTreeReader::Notify()
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Tell readers we now have a tree.
-/// fValues gets insertions during this loop (when parameterized arrays are read),
+/// fValues gets insertions during this loop (when parametrized arrays are read),
 /// invalidating iterators. Use old-school counting instead.
 
 Bool_t TTreeReader::SetProxies() {
@@ -521,7 +521,7 @@ TTreeReader::EEntryStatus TTreeReader::SetEntryBase(Long64_t entry, Bool_t local
          // don't try to load entries anymore. Can happen in these cases:
          // while (tr.Next()) {something()};
          // while (tr.Next()) {somethingelse()}; // should not be calling somethingelse().
-         fEntryStatus = kEntryNotFound;
+         fEntryStatus = kEntryBeyondEnd;
          return fEntryStatus;
       }
       if (entry >= 0) {
@@ -574,7 +574,7 @@ TTreeReader::EEntryStatus TTreeReader::SetEntryBase(Long64_t entry, Bool_t local
                value->NotifyNewTree(fTree->GetTree());
             }
          }
-         fEntryStatus = kEntryNotFound;
+         fEntryStatus = kEntryBeyondEnd;
          return fEntryStatus;
       }
 
